(define-module (alterator vm option)
  :use-module (srfi srfi-1)
  :use-module (srfi srfi-2)
  :use-module (srfi srfi-11)
  :use-module (srfi srfi-13)
  :use-module (alterator vm defs)
  :use-module (alterator vm utils)
  :export (task-options))

(define (probe-option option)
  (format #t "PROBE  ---------------------------------------------------------\n~S\n" option)
  option)

(define (set-task-option! name value)
  (and-let*
   ((task (fluid-ref current-task)))
   (set-procedure-property!
    task
    'context
    (assv-set! (procedure-property task 'context)
               name value))))

(define (task-option name)
  (and-let*
   ((task (fluid-ref current-task))
    (options (procedure-property task 'context)))
   (assv-ref options name)))

(define (set-task-option-transform! name value)
  (and-let*
   ((task (fluid-ref current-task)))
   (set-procedure-property!
    task
    'transform
    (assv-set! (procedure-property task 'transform)
               name value))))

(define (option-importance< opta optb)
  (and
   (or (memq 'important (assv-ref opta 'flags))
       (memq 'not-required (assv-ref optb 'flags))) #t))

(define (filter-option:custom option)
  (and option
       (case (fluid-ref current-operation)
         ((op-pt-create)
          ;; no OS/2 selection, we're linux
          #f)
         ((op-segment-create)
          ;; TypeByName should be enough
          (cond ((equal? (assv-ref option 'name) "TypeByNumber")
                 (set-task-option! 'type-by-number (assv-ref option 'value)) #f)
                ((equal? (assv-ref option 'name) "Bootable") #f)
                (#t option)))
         ((op-volume-mkfs)
          (cond ((equal? (assv-ref option 'name) "version") #f)
                (#t option)))
         (else option))))

(define (filter-option:active option)
  (and option
       (if (memq 'inactive (assv-ref option 'flags)) #f
           option)))

(define (fourth-primary? seg-or-disk)
  (let* ((disk (if (eqv? 'segment (obj->type seg-or-disk))
                   (car (obj->children seg-or-disk))
                   seg-or-disk))
         (segs (obj->parents disk)))
    (and (< (length (filter segment-meta? segs)) 2)
         (< 2 (length (filter segment-data? segs))))))

(define (mangle-option:hooks option)
  (case (fluid-ref current-operation)
    ((op-volume-mount op-volume-setmntent)
     (alist-delete 'guards (alist-delete 'hooks option)))
    (else option)))

(define (mangle-option:custom option)
  (cond
   ((member (assv-ref option 'name) '("Size" "Offset"))
    (assv-set! option 'flags `(important . ,(assv-ref option 'flags))))
   ((equal? (assv-ref option 'name) "Primary")
    (cond ((and (assv-ref option 'value)
                (not (task-option 'force-extended))
                (fourth-primary? (fluid-ref current-object)))
           (set! ((fluid-ref current-task) 'options) '(("Primary" . #f)))
           (set-task-option! 'force-extended #t)
           (assv-set! option 'value #f))
          ((not (assv-ref option 'value))
           (assv-set! option 'flags `(inactive . ,(assv-ref option 'flags))))
          (#t option)))
   ((equal? (assv-ref option 'name) "TypeByName")
    (set-task-option-transform!
     'type-by-name
     (lambda args
       (cond ((equal? "Linux RAID" (assoc-ref args "TypeByName"))
              (acons "TypeByNumber" 253 (alist-delete "TypeByName" args)))
             ((equal? "PReP" (assoc-ref args "TypeByName"))
              (acons "Bootable" #t (acons "TypeByNumber" 65 (alist-delete "TypeByName" args))))
             (#t args))))
    (let ((option
           (assv-set! option 'constraint
                      (reverse (append '("PReP" "Linux RAID")
                                       (cdr (reverse (delete "HPFS" (assv-ref option 'constraint)))))))))
      (cond ((equal? (task-option 'type-by-number) "253")
             (assv-set!
              (assv-set! option 'value "Linux RAID")
              'flags `(lasthop . ,(assv-ref option 'flags))))
            ((equal? (task-option 'type-by-number) "65")
             (assv-set!
              (assv-set! option 'value "PReP")
              'flags `(lasthop . ,(assv-ref option 'flags))))
            ((equal? (assv-ref option 'value) "Linux LVM")
             (assv-set! option 'flags `(lasthop . ,(assv-ref option 'flags))))
            (#t option))))
   (#t option)))

(define (mangle-option:value option)
  (let ((name (assv-ref option 'name))
        (value (assv-ref option 'value)))
    (case (assv-ref option 'widget)
      ((edit)
       (assv-set! option 'value
                  (if (number? value)
                      (number->string value)
                      (or value ""))))
      ((slider)
       (let ((constraint (assv-ref option 'constraint)))
         (cond ((eqv? 'sector (assv-ref option 'unit))
                (set-task-option-transform!
                 'scale-sectors
                 (lambda args
                   (let ((val (assoc-ref args name)))
                     (if val (assoc-set! args name (* 2048 val))
                         args))))
                (let ((sect->mb (lambda(v) (round-integer (/ v 2048)))))
                  (assv-set!
                   (assv-set!
                    (assv-set! option 'constraint (map sect->mb constraint))
                    'value (sect->mb value))
                   'unit 'MB)))
               (#t option))))
      (else option))))

(define (mangle-option:pixmap option)
  (acons 'pixmap
         (if (memq 'not-required (assv-ref option 'flags))
             ""
             "reqfield.gif")
         option))

(define (mangle-option:widget option)
  (acons 'widget
         (cond
          ((eq? 'boolean (assv-ref option 'type))
           'checkbox)
          ((eq? 'range (assv-ref option 'constraint-type))
           'slider)
          ((eq? 'list (assv-ref option 'constraint-type))
           (if (memq 'value-is-list (assv-ref option 'flags))
               'checklist
               (if (memq 'expandable (assv-ref option 'flags))
                   'combobox/ex
                   'combobox)))
          (#t 'edit))
         option))

(define (mangle-option:flags option)
  (let ((flags (assv-ref option 'flags)))
    (acons 'ready (or (and (memq 'not-required flags) #t)
                      (not (memq 'no-initial-value flags)))
           option)))

(define (mangle-option:constraint option)
  (let ((constraint (assv-ref option 'constraint)))
    (acons 'type (car constraint)
           (acons 'constraint-type (cadr constraint)
                  (acons 'constraint (caddr constraint)
                         (alist-delete 'constraint option))))))

(define (mangle-option:name option)
  (acons 'name (car option) (cdr option)))

(define (mangle-option option)
  (filter-option:custom
   (filter-option:active
    (mangle-option:hooks
     (mangle-option:custom
      (mangle-option:value
       (mangle-option:pixmap
        (mangle-option:widget
         (mangle-option:flags
          (mangle-option:constraint
           (mangle-option:name option)))))))))))

(define (task-options op obj task)
  (with-fluids
   ((current-operation op)
    (current-object obj)
    (current-task task))
   (sort
    (filter-map mangle-option (task 'options))
    option-importance<)))
