(define-module (alterator vm utils)
  :use-module (srfi srfi-1)
  :use-module (srfi srfi-2)
  :export (obj->obj
           obj->name
           obj->size
           obj->start
           obj->type
           obj->ctype
           obj->dtype
           obj->plugin
           obj->plugin->name
           obj->children
           obj->parents
           obj->producer
           obj->flags
           
           disk?
           segment?
           segment-data?
           segment-meta?
           segment-free?
           container?
           raid?
           lvm?
           compat?
           evms?

           name-of
           value-of
           tree-of))

;---------------------------------------------------------------
(define (obj->obj obj)
  (assv-ref (obj) 'object))

(define (obj->name obj)
  (assv-ref (obj) 'name))

(define (obj->size obj)
  (assv-ref (obj) 'size))

(define (obj->start obj)
  (case (obj->type obj)
    ((segment) (assv-ref (obj) 'start))
    ((volume) (obj->start (obj->obj obj)))
    (else -1)))

(define (obj->type obj)
  (and (procedure? obj)
       (procedure-property obj 'type)))

(define (obj->ctype obj)
  (and (compat? obj)
       (obj->type (obj->obj obj))))

(define (obj->dtype obj)
  (or (assv-ref (obj) 'data-type)
      (and (compat? obj)
           (assv-ref ((obj->obj obj)) 'data-type))))

(define (obj->plugin obj)
  (let ((o (obj)))
    (or (assv-ref o 'plugin)
        (assv-ref o 'fsim))))

(define (obj->plugin->name obj)
  (and-let* ((plugin (obj->plugin obj)))
            (obj->name plugin)))

(define (obj->children obj)
  (assv-ref (obj) 'children))

(define (obj->parents obj)
  (assv-ref (obj) 'parents))

(define (obj->producer obj)
  (assv-ref (obj) 'producer))

(define (obj->flags obj)
  (assv-ref (obj) 'flags))

;---------------------------------------------------------------
(define (compat? obj)
  (memq 'compatibility (obj->flags obj)))

(define (disk? obj)
  (case (obj->type obj)
    ((volume)
     (eqv? (obj->ctype obj) 'disk))
    ((disk) #t)
    (else #f)))

(define (segment? obj)
  (case (obj->type obj)
    ((volume)
     (eqv? (obj->ctype obj) 'segment))
    ((segment) #t)
    (else #f)))

(define (segment-data? obj)
  (eqv? 'data (obj->dtype obj)))

(define (segment-meta? obj)
  (eqv? 'meta (obj->dtype obj)))

(define (segment-free? obj)
  (eqv? 'free (obj->dtype obj)))

(define (raid? obj)
  (case (obj->type obj)
    ((volume) (and (compat? obj) (raid? (obj->obj obj))))
    ((region)
     (and (member (obj->plugin->name obj)
                  '("MDRaid0RegMgr" "MDRaid1RegMgr" "MDRaid5RegMgr")) #t))
    (else #f)))

(define (lvm? obj)
  (case (obj->type obj)
    ((volume) (and (compat? obj) (lvm? (obj->obj obj))))
    ((region) (and (string=? (obj->plugin->name obj) "LVM2") #t))
    (else #f)))

(define (evms? obj)
  (and (eqv? (obj->type obj) 'volume)
       (not (compat? obj))))

(define (container? obj)
  (eqv? (obj->type obj) 'container))

;---------------------------------------------------------------
(define (name-of thing) (thing 'name '()))
(define (value-of thing) (thing 'value '()))
(define (tree-of thing) (thing 'tree '()))
