;  -*- scheme -*-

(document:surround "/std/frame")
(document:insert "/std/functions")

(document:insert "/vm/functions")

;----------------------------------------------------------------

(define (evms-open)
  (woo-catch/message
   (lambda()
     (woo-write "/evms/control" 'control 'open 'installer #t)
     (woo-write "/evms/control" 'control 'update))))

(define (current-profile)
  (string-append
   "/evms/profiles/"
   (car
    ((if (eqv? 2 (rl current)) reverse values)
     (woo-list-names "/evms/profiles")))))

(define (clearall-state)
  (eqv? 1 (rl current)))

(define (commit-state)
  (not (eqv? 2 (rl current))))

(define (profile-apply)
  (woo 'apply (current-profile)
       'commit (commit-state)
       'clearall (clearall-state)
       'exclude '()))

(define (profile-commit . args)
  (evms-with-splash
   (lambda()
     (woo-catch/message
      (lambda ()
        (woo-write "/evms/control" 'control 'commit)
        (apply woo-write "/evms/control" 'control 'close args))))))

;---------------------------------------------------------------
(define volume-row-alist
  `((name . ,(_ "Name"))
    (fsim . ,(_ "Filesystem"))
    (size . ,(_ "Size [free]"))))

(define (make-vol-header)
  (map label
       (map (lambda (s) (string-append "<b>" s "</b>"))
	    (map cdr volume-row-alist))))

(define (make-vol-rows)
  (let ((props (map car volume-row-alist))
        (volumes (woo-list-names "/evms/storage/volumes")))
    (append-map
     (lambda (vol)
       (let-values
	   (((name fsim size)
	     (apply values
		    (woo-leaf-props
		     (string-append "/evms/storage/volumes/" vol)
		     props))))
	 (list
	  (label name)
	  (label (if (pair? fsim) (car fsim) fsim))
	  (label size))))
     volumes)))

(define (update-volumes)
  (clean-widget
   volumes-box
   (lambda()
     (apply volumes-box
	    (make-vol-header)
	    (separator)
	    (separator)
	    (separator)
	    (make-vol-rows)))))
    
(define action-row-alist
  `((default . ,(_ "Use unpartitioned disk space"))
    (clearall . ,(_ "Destroy all partitions, then autopartition"))
    (custom . ,(_ "Custom install"))))

(define (warn-on-clearall)
  (eqv? 'ok (document:popup-warning
             (string-append
              "<center><p><b>"
              (_ "All data from all your disks will be lost! Continue?")
              "</b></p></center>")
             'cancel 'ok)))

;---------------------------------------------------------------
(groupbox
 (_ "Volumes")
 (document:id volumes-box (gridbox columns "30;30;40")))

(groupbox
 (_ "Select installation method")
 (document:id
  rl
  (radiolistbox rows (map cdr action-row-alist) current 0)))

(frame:on-next
 (lambda ()
   (if (and (eqv? (rl active) 1)
            (not (warn-on-clearall)))
       'cancel
       (let ((reply (profile-apply)))
         (format #t "<* ~S\n" reply)
         (cond
          ((eqv? (caar reply) 'nexthop)
           (frame:replace (cadar reply) 'installer #t 'nexthop '/vm/blonde)
           'cancel)
          ((eqv? (caar reply) #t)
           (let loop ((ret (profile-commit)))
             (format #t "RET: ~S\n" ret)
             (cond ((null? ret)             ; all went good
                    #t)
                   ((not ret)               ; fail
                    (woo-write "/evms/control" 'control 'reset)
                    'cancel)
                   (#t                      ; show warn
                    (case (document:popup-warning (cadar ret) 'cancel 'ok)
                      ((ok)                 ; ignore this warning
                       (loop (profile-commit 'ignore (caar ret))))
                      ((cancel)             ; let user fix things
                       (frame:replace '/vm/table 'installer #t)
                       'cancel))))))
          (#t
           (woo-write "/evms/control" 'control 'reset)
           (woo-write "/evms/control" 'control 'update)
           (document:popup-critical
            (apply string-append
                   "<center><p><b>"
                   (_ "Unable to autopartition, not enough disk space")
                   "</b></p></center>"
                   (map cadr reply)) 'ok)
           'cancel))))))

(document:root
 (when loaded
   (and (evms-open)
        (if (or (eqv? (global 'frame:direction) 'next)
                (global 'lemmein))
            (evms-with-splash
             (lambda()
	       (update-volumes)
               (frame:back-activity #f)))
            (frame:replace '/vm/table 'installer #t)))))
