; $Id: option.scm,v 1.17 2006/10/02 17:09:33 me Exp $

(document:surround "/std/base")
(document:insert "/std/functions")

(document:insert "/vm/functions")

(document:envelop with-attributes
                  (changed-value get-value renew-value commit-value))

(define value-widget (make-widget "box"))

;---------------------------------------------------------------
(define current-option (make-cell #f))
(define option-setter (make-cell #f))
(define value-ready (make-cell #f))
(define reset-edit (make-cell #f))
(define discard-renewal (make-cell #f))

(define (option->name)
  (assv-ref (cell-ref current-option) 'name))

(define (option->value)
  (assv-ref (cell-ref current-option) 'value))

(define (option->values)
  (assv-ref (cell-ref current-option) 'constraint))

(define (edit-callback current-value)
  (if (not (and (cell-ref value-ready)
                (not (string-null? current-value))))
      (begin
        (cell-set! value-ready (not (cell-ref value-ready)))
        (op-value changed-value))))

(define (make-option-setter option)
  (let ((name (assv-ref option 'name)))
    (lambda (current)
      (format #t "SET-VALUE! ~S: ~S\n" name current)
      (let ((effect (woo-write-task 'set-options! name current)))
        (format #t "EFFECT: ~S ~S\n" name effect)
        (cond ((not (caar effect))      ; error from backend
               (format #t "AIEEE! ~S\n" (cadar effect))
               (document:popup-critical (_ (cadar effect)) 'ok)
               (cell-set! reset-edit #t)
               (op-value get-value)     ; reset to last known state
               #f)                      ; and not let pass further
              ((null? (cdar effect))    ; no effect, safe to continue
               #t)
              (#t                       ; options/objects changed, reload
               (op-value get-value)     ; no way further
               (cell-ref discard-renewal)))))))

(define (make-checkbox)
  `(,type checkbox
          ,state ,(option->value)
          ,(when toggled ((cell-ref option-setter) (op-value state)))
          ,(when commit-value #t)       ; committed inplace
          ,(when renew-value
             ;;(format #t "RENEW checkbox: ~S:~S\n" (option->name) (option->value))
             (op-value state (option->value)))))

(define (make-checklist)
  (let ((get-rows (lambda()
                    (let ((val (option->value))
                          (vals (option->values)))
                      (if (null? val) vals
                          (map (lambda (v) (if (member v val) v (cons v #f))) vals))))))
    (letrec ((cl (checklist
                  checklist-rows (get-rows)
                  (when clicked ((cell-ref option-setter) (cl checklist-active-rows)))
                  )))
      `(,type vbox ,cl
              ;;,(when clicked ((cell-ref option-setter) (op-value checklist-active-rows)))
              ,(when commit-value #t)   ; committed inplace
              ,(when renew-value
                 ;;(format #t "RENEW checklist: rows: ~S\n" (get-rows))
                 (op-value rows (get-rows)))))))

(define (make-combobox . editable)
  (let ((get-all (lambda() (option->values)))
        (get-cur (lambda() (let ((val (option->value)))
                             (or (list-index (lambda (x) (equal? x val))
                                             (option->values))
                                 0)))))
    `(,type combobox ,alterability ,(not (null? editable))
            ,rows ,(get-all) ,current ,(get-cur)
            ,(when selected ((cell-ref option-setter) (op-value current-text)))
            ,(when commit-value (or (null? editable) ((cell-ref option-setter) (op-value current-text))))
            ,(when renew-value
               ;;(format #t "RENEW: ~S rows: ~S current: ~S\n" (option->name) (get-all) (get-cur))
               (op-value rows (get-all) current (get-cur))))))

(define (make-slider)
  (let ((val (option->value))
        (vmin (car (option->values)))
        (vmax (cadr (option->values)))
        (vstep (caddr (option->values))))
    `(,type "spinbox"
            ,(label (or (assv-ref (cell-ref current-option) 'unit) ""))
            ,minimum ,vmin
            ,maximum ,vmax
            ,value ,val
            ,step ,vstep
            ,orientation horizontal
            ,expanded #t
            ,(when changed
               ;;(format #t "CHANGED spinbox: ~S ~S\n" (option->value) (option->values))
               ((cell-ref option-setter) (op-value value)))
            ,(when commit-value #t) ; committed inplace
            ,(when renew-value
               (format #t "RENEW slider: ~S ~S\n" (option->value) (option->values))
               (op-value minimum (car (option->values))
                         maximum (cadr (option->values))
                         value (option->value))))))

(define (make-edit)
  (let ((val (option->value)))
    `(,type edit ,text ,val
            ,(when changed (edit-callback (op-value text)))
            ,(when commit-value
               (cell-set! discard-renewal #t)
               (or (string-null? (op-value text)) ((cell-ref option-setter) (op-value text))))
            ,(when renew-value
               ;;(format #t "RENEW edit: ~S ~S\n" (cell-ref reset-edit) (option->value))
               (and (cell-ref reset-edit)
                    (not (string-null? (option->value)))
                    (op-value text (option->value)))))))

(document:envelop
 with-proxy-attributes                                 ; TODO option getter
 ((option ((proxy-set self opt)
           ;;(format #t "---------------------------------------------------------------\n")
           ;;(pretty-print opt)
           ;;(format #t "---------------------------------------------------------------\n")
           (cell-set! current-option opt)
           (cell-set! option-setter (make-option-setter opt))
           (cell-set! value-ready (assv-ref opt 'ready))
           (op-title text (_ (assv-ref opt 'title)))
           (op-pixmap pixmap (assv-ref opt 'pixmap))
           ;;(op-value tooltip (or (assv-ref opt 'tip) ""))
           (apply op-value
                  (case (assv-ref opt 'widget)
                    ((checkbox) (make-checkbox))
                    ((checklist) (make-checklist))
                    ((combobox) (make-combobox))
                    ((combobox/ex) (make-combobox #t))
                    ((slider) (make-slider))
                    ((edit) (make-edit))))))

  (set-value! ((proxy-set self opts)
               (cell-set! reset-edit #t) ; renew request from other option
               (any
                (lambda (opt)
                  (and (equal? (assv-ref opt 'name)
                               (assv-ref (cell-ref current-option) 'name))
                       (begin (cell-set! current-option opt)
                              (cell-set! option-setter (make-option-setter opt))
                              (op-value renew-value) #t)))
                opts)))

  (has-value? ((proxy-get self)
               (cell-ref value-ready)))

  (apply-value ((proxy-get self)
                (op-value commit-value)))

  (got-value ((proxy-set self callback)
              (op-value changed-value callback)))

  (need-value ((proxy-set self callback)
               (op-value get-value callback)))))

type 'gridbox columns "5;40;55"
(document:id op-pixmap (label ""))
(document:id op-title  (label ""))
(document:id op-value (value-widget))
